// Comp 1672 
// Singly-linked list for Lab 8
#include <iostream>
#include "LinkedList.h"
#include "Node.h"

// default constructor
LinkedList :: LinkedList() :
	myHead(NULL)
{
}

// copy constructor
LinkedList :: LinkedList(const LinkedList &orig) :
	myHead(NULL)
{
	// this copy constructor does a DEEP copy, it copies the
	// values in the pointers, not just the pointers themselves
	Node *temp = orig.myHead;
	while(temp != NULL)
	{
		pushBack(temp->data());
		temp = temp->next();
	}	
}

// assignment operator
LinkedList& LinkedList :: operator=(const LinkedList &orig)
{
   // this does a DEEP copy, it copies the
   // values in the pointers, not just the pointers themselves
   Node *temp = orig.myHead;
   while(temp != NULL)
   {
        pushBack(temp->data());
        temp = temp->next();
   }	
}

// destructor
LinkedList :: ~LinkedList()
{
	// avoid memory leaks, clean up all memory!
	Node *CurNode = myHead;
	Node *tempNode = NULL;
        // traverse entire list, delete each element
	while(CurNode != NULL)
	{
		tempNode = CurNode->next();
		delete CurNode;
		CurNode=tempNode;
	}
        myHead = 0;
}

// put this new data element onto the end of the list
void LinkedList :: pushBack(int newData)
{
   // create new node and initialize its data
   Node *newNode = new Node;
   newNode->data(newData);
   if (!myHead) {
       // list is empty
       myHead = newNode;
   }
   else
   {
        // find the end of the list, and tack this new node onto the end
	Node *tail(getTail());
        tail->next(newNode);
   }
}

int LinkedList :: popBack()
{
	int returnValue;
        // handle case of empty list
	if(!myHead)
		return -1;

        // find the end of the list, and see what data is there
	Node *tail(getTail());
        returnValue = tail->data();

        if (myHead == tail) {
	    // there's just one thing in the list - now it will be empty
	    myHead = 0;		
	} else {
	    // the second-to-last element is now the tail
	    Node *prevNode(getPrevious(tail));
	    prevNode->next(0);
	}
        // release memory for this now-defunct node
	delete tail;
        // send back the data that was stored at this now-defunct node
	return returnValue;
}

// removes node passed as parameter
void LinkedList::remove(Node *deadNode)
{
    Node *temp;
    Node *tail(getTail());
    if (deadNode == myHead) {   	// case 1 - get rid of the first element
	temp = myHead;
	myHead = myHead->next();
	delete temp;        	
    } else if (deadNode == tail) {	// case 2 - get rid of the last element
	temp = getPrevious(tail);
	temp->next(0);
	delete temp;
    } else {				// case 3 - get rid of one in the middle
	temp = getPrevious(deadNode);
	temp->next(0);
	delete deadNode;
    }
}

// traverse the list and find the node BEFORE the one passed
Node * LinkedList::getPrevious(Node *n) const
{
    Node *temp = myHead;
    if (n == myHead) {
         return 0;   // there IS NO previous node
    }
    // traverse the list until we find a node pointing to n
    while (temp->next() != n)
	temp = temp->next();
    return temp;
}

// find the last element in the list
Node *LinkedList::getTail() const
{
    Node *temp = myHead;
    if (!temp) return 0;
    while (temp->next() != 0)
	temp = temp->next();
    return temp;
}

// traverse the list, dislaying the data in each element
void LinkedList::display() const
{
    Node *temp = myHead;

    cout<<"List Data: (Head) ";

    while(temp != 0)
    {
	cout<<temp->data()<<" -> ";
  	temp=temp->next();
    }
    cout<<"NULL (Tail)"<<endl;
}

void LinkedList::insert(int newData, Node *n)
{
    Node *newNode, *temp;
    newNode = new Node;
    newNode->data(newData);
    temp = n->next();
    n->next(newNode);
    newNode->next(temp);
}

Node * LinkedList::head() const
{
    return myHead;
}
